/*****************************************************************************
 *  Module for Microchip Graphics Library
 *  Epson. S1D13517 LCD controller driver
 *  to be used with GFX 3e PICtail board
 *****************************************************************************
 * FileName:        S1D13517.c
 * Dependencies:    Graphics.h
 * Processor:       PIC24, PIC32
 * Compiler:        MPLAB C30, MPLAB C32
 * Linker:          MPLAB LINK30, MPLAB LINK32
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * Copyright  2008 Microchip Technology Inc.  All rights reserved.
 * Microchip licenses to you the right to use, modify, copy and distribute
 * Software only when embedded on a Microchip microcontroller or digital
 * signal controller, which is integrated into your product or third party
 * product (pursuant to the sublicense terms in the accompanying license
 * agreement).  
 *
 * You should refer to the license agreement accompanying this Software
 * for additional information regarding your rights and obligations.
 *
 * SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
 * KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
 * OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR
 * PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR
 * OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION,
 * BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT
 * DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL,
 * INDIRECT, PUNITIVE OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
 * COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY
 * CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
 * OR OTHER SIMILAR COSTS.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Adam Folts     05/28/10
 thk 12/30/10 added 102x64 low level KibaCorp drivers and delay code
 thk 1/22/11 added red backlight control
 thk 2/19/2011 added PWM to backlight for green added RED and GREEN LED controls
 JP3 pin 5 RF4(31) is red Led
 JP3 pin 4 RB3 (13) is red back light control
 JP4 pin 1 RD10/IC3/(44) is green led
 JP4 pin 2 Rd4/CN13 (52) green back light control
 
 *****************************************************************************/
#include "Graphics\Graphics.h"
#include "S1D13517.h"

// Color
WORD    _color;

// Clipping region control
SHORT   _clipRgn;
// Left clipping region border
SHORT    _clipLeft;

// Top clipping region border
SHORT    _clipTop;

// Right clipping region border
SHORT    _clipRight;

// Bottom clipping region border
SHORT    _clipBottom;
#define CS   _LATB0    // CS output to EADOG102
#define RESET  _LATB1   // RESET output EADOG102
#define A0   _LATB2   // Ao output to EADOG102 CMD/data
#define REDBACKLT _LATB3  //red blacklight control

#define greenLED _LATD10  //on board green led
#define redLED _LATF4   //on board red led
#define HRES 102
#define VRES 64


#define RED8(color16)   (BYTE) ((color16 & 0xF800) >> 8)
#define GREEN8(color16) (BYTE) ((color16 & 0x07E0) >> 3)
#define BLUE8(color16)  (BYTE) ((color16 & 0x001F) << 3)
#define TRANSPARENTCOLOR  RGB565CONVERT(50, 20, 13)   //Transparent Color chosen for the application
#define WAIT_UNTIL_FINISH(x)    while(!x)
//variables for EXP32 Drivers **************************************************thk 12/30/10**********
//no font
int YIN[102];  //VMAP
#define X0 0
#define Y0 (VRES/2)
#define iend 500
#define jend 1000
int xus;
int yus;
float xf;
float  yf;
unsigned char  Vmap[HRES][VRES/8];  // array 102 bytes by 8 bytes or 816 bytes ***change 102-6
// Prototypes 
void initSD( void);
void clearScreen (void);
unsigned char writeSPI( unsigned char b);
void clearDisplay(void);
void dumpVmap( void);
void clearVmap( void);
void Plot (unsigned char x, unsigned y);
void goto_xy(char x, char y);
void enableCS(void);
void disableCS(void);
void setAO(void);
void clearA0(void);
void Delayms( unsigned t);
void delay_5us(void);
void redBLON(void);
void redBLOFF(void);
void greenBLOFF(void);
void greenBLON( int setting);
void greenLEDON(void);
void greenLEDOFF(void);
void redLEDON(void);
void redLEDOFF(void);


//********************EXP32 SPI and timer/delay Drivers for 102x64  Begin***********************12/30/10******************
const char init[] = {0x40,    //Display start line 0 (6)
                     0xa1,    //SEG reverse (13)
                     0xc0,    //Normal COM0...COM63 (14)
                     0xa4,	  // disable -> set all pixel to ON(10)
                     0xa6,    //Display inverse off (11)
                     0xa2,    //Set Bias 1/9 (Duty 1/65) (17)
                     0x2f,    //Booster, Regulator and Follower On (5)
                     0x27,    //set constrast (8)
                     0x81,	  // set volume (9)
                   //  0x10,    //
                    0x08,	//optimize setting for green/yellow
                     0xfa,	  // set temp compensation (25)
                     0x90,
                     0xaf,    //Display on (12)
                     };

void initSD( void)
{
	AD1PCFG = 0xFFFFFFFF; //added for EXP 32
    DDPCON=0x00; //added fOR EXP 32 to turn off JTAG
    // added OC3 and Timer3 for PWM led green backlight
OpenTimer3(T3_ON |T3_PS_1_1, 0xFFFF);
OpenOC5(OC_ON | OC_TIMER3_SRC |OC_TIMER_MODE16| OC_PWM_FAULT_PIN_DISABLE, 0x7fff, 0x7fff);

    
   int i=0;
 
   	CS =1;
 
   A0=0;					//this must be equivalent of CD (=0) for command
 
   RESET =1;
   REDBACKLT =0;	//turn off backlight
   redLED =0;
   greenLED =0;
mPORTBSetPinsDigitalOut (BIT_3| BIT_2 |BIT_1 | BIT_0 );
mPORTDSetPinsDigitalOut ( BIT_10); //red led
mPORTFSetPinsDigitalOut	(BIT_4);  //green led
    Delayms(50);  //10 msec
    RESET =0;
    Delayms(50); //10 msec
 
     RESET =1;
   
   
    // init the spi module for a slow (safe) clock speed first
OpenSPI2(SPI_MODE8_ON|CLK_POL_ACTIVE_LOW| ENABLE_SDO_PIN|SPI_CKE_ON|SPI_SMP_OFF|MASTER_ENABLE_ON|SEC_PRESCAL_1_1|PRI_PRESCAL_1_1,SPI_ENABLE);
SPI2BRG =15; //at PB =40 Mhz this is 1.25MHZ
//	SPI2CON1 = 0x013F;  // CKE=1; CKP=0, sample middle, prescale 1:1

//    SPI2STAT = 0x8000;  // enable the peripheral
     	CS =0;
    //initialize and enable the EADOG128
    
   
   
    for (i =0; i<sizeof init; i++)
    {
	    writeSPI(init[i]);
	    //delay_5us(); //not needed
	    
	   } 

	clearScreen();
// debug capability
			
	//draw a broken byte line stream				
//	   goto_xy(0,3);
//	  	setAO();			//CD =1 or data
//	   for (i=0; i<64;i++)
//	   writeSPI(0x55);
//	   for (i=0; i<64;i++)
//	   writeSPI(0xaa);
}   // initSD

void clearScreen (void){
	
	clearVmap();
		dumpVmap();	
	
	
	
}	
// send one byte of data and receive one back at the same time
unsigned char writeSPI( unsigned char b)
{
    SPI2BUF = b;					// write to buffer for TX
    while( !SPI2STATbits.SPIRBF);	// wait for transfer to complete
 
    return SPI2BUF;    				// read the received value
}// writeSPI

void clearDisplay(void) {
	
	 //clear all display ram
	   int y=0;
	   int i=0;
	    for (y =0;y<8;y++)
	 {
	   goto_xy(0,y);
	   setAO();		//data mode
	   //	for (i=0; i<128;i++) ****change for 102-6
	   	 	for (i=0; i<102;i++)
	   	writeSPI(0x00); 
		}							

}
void clearVmap( void) {
	int y=0;
	int x=0;
	   // for (x =0;x<128;x++) //**change for 102-6
	    for (x =0;x<102;x++)
	    	for (y=0; y<8; y++)
	    	{
		    Vmap [x][y] =0;
				}
		    	
  }	
	
void dumpVmap( void)
{
	//dump Vmap onto display
	   int y=0;
	   int x=0;
	    for (y =0;y<8;y++)
	 {
	   goto_xy(0,y);
	   setAO();		//data mode
	 //  	for (x=0; x<128;x++) //****change for 102-6
	 	for (x=0; x<102;x++)
	   	writeSPI(Vmap[x][y]); 
		}				
	
	
}
//plots pixel point into processor ram x =0 to 102 y =0 to 63 ***change 102-6
void Plot (unsigned char x, unsigned y) 
{
	if ((x<HRES)&& (y<VRES)) {	
	unsigned char temp = 0;
	temp = Vmap[x][y/8];  //retrive byte
	if (GetColor()==BLACK) {
	temp =temp| (0x01 << (y%8))	; //or appropiate bit
	}
	if (GetColor()==WHITE) {
	temp =temp& (~(0x01 << (y%8)));	// and appropiate bit
	}
	Vmap[x] [y/8] = temp;			//write back
	}
}			
	
	
		

// stuff lifted from EADOG driver*************************************

void goto_xy(char x, char y)
{
  char set[] = {0xb0, 0x10, 0x00};    //Befehlsarray
  set[0] = set[0] + y;                //Y - Position auf Steuerbefehl addieren
  set[2] = set[2] + (x & 0x0f);       //Low - Nibble auf Steuerbefehl addieren
  set[1] = set[1] + (x >> 4);         //High - Nibble auf Steuerbefehl addieren
 
   	clearA0();			//cmd mode
	writeSPI(set[0]);	//(7) page address
	writeSPI(set[2]);	// (4) low LSB first ***change for 102-6
	writeSPI(set[1]);	// (4) high MSB second *** this order is opposite to 128-6

}


//*************end of EADOG Driver**********************************

void enableCS(void) {
		CS =0;
	
}	

void disableCS(void) {
		CS =1;
	
}	
void setAO(void) {  //equivalent of CD command/data 1 =data
		A0 =1;
	
}	

void clearA0(void) {//equivalent of CD command/data 0 =command
		A0=0;
	
}


void redBLON(void){
	
	REDBACKLT =1;
}	
void redBLOFF(void){
	REDBACKLT =0;
}
void greenBLOFF(void){
 SetDCOC5PWM(0); 	
	
}	
void greenBLON( int setting){
 SetDCOC5PWM(setting); 	
	
}	
void greenLEDON(void){
	greenLED =1;
}	
void greenLEDOFF(void){
	greenLED =0;
}	
void redLEDON(void){
redLED =1;	
	
}	
void redLEDOFF(void){
redLED =0;	
}		
	
	
void Delayms( unsigned t)
{
    T1CON = 0x8000;     // enable tmr1, Tcy, 1:1
    while (t--)
    {
        TMR1 = 0;
        while (TMR1<16000);
      
    }
} // Delayms
	void delay_5us(void) { //modified for PIC32 @80mhz
		int j;
			for (j=0;j<46; j++); 
	}
	
//**************************************EXP32 102x64 Drivers END*********************************************
/////////////////////// LOCAL FUNCTIONS PROTOTYPES ////////////////////////////
void        SetAddress(DWORD address);
void        SetReg(BYTE index, BYTE value);
BYTE        GetReg(BYTE index);
WORD        WriteBuffer[8] = {TRANSPARENTCOLOR,TRANSPARENTCOLOR,TRANSPARENTCOLOR,TRANSPARENTCOLOR,TRANSPARENTCOLOR,TRANSPARENTCOLOR,TRANSPARENTCOLOR,TRANSPARENTCOLOR};

#ifdef USE_DRV_PUTIMAGE
void        PutImage1BPP(SHORT left, SHORT top, FLASH_BYTE *bitmap, BYTE stretch);
void        PutImage4BPP(SHORT left, SHORT top, FLASH_BYTE *bitmap, BYTE stretch);
void        PutImage8BPP(SHORT left, SHORT top, FLASH_BYTE *bitmap, BYTE stretch);
void        PutImage16BPP(SHORT left, SHORT top, FLASH_BYTE *bitmap, BYTE stretch);

void        PutImage1BPPExt(SHORT left, SHORT top, void *bitmap, BYTE stretch);
void        PutImage4BPPExt(SHORT left, SHORT top, void *bitmap, BYTE stretch);
void        PutImage8BPPExt(SHORT left, SHORT top, void *bitmap, BYTE stretch);
void        PutImage16BPPExt(SHORT left, SHORT top, void *bitmap, BYTE stretch);
#endif


/*********************************************************************
* Macro:  WritePixel(color)
* PreCondition: none
* Input: color 
* Output: none
* Side Effects: none
* Overview: writes pixel at the current address
* Note: chip select should be enabled
********************************************************************/
#ifdef USE_16BIT_PMP
#define WritePixel(color)	DeviceWrite(color)
#else
#ifdef USE_PALETTE
#define WritePixel(color)	DeviceWrite(color)
#else
#define WritePixel(color)	{ DeviceWrite(((WORD_VAL)color).v[1]); DeviceWrite(((WORD_VAL)color).v[0]);}
#endif
#endif
#define PMPWaitBusy()   while(PMMODEbits.BUSY); 
void SetAddress(DWORD address){}
//device calls

/*********************************************************************
* Function:  void  SetReg(WORD index, BYTE value)
* PreCondition: none
* Input: index - register number
*        value - value to be set
* Output: none
* Side Effects: none
* Overview: sets graphics controller register (byte access)
* Note: none
********************************************************************/
void SetReg(BYTE index, BYTE value)
{
 
    DeviceSetCommand(); // set RS line to low for command
    DeviceSelect();     // enable S1D13517
       
    PMDIN1 =  index;    // Tell Epson which Register to write to
    PMPWaitBusy();

    DeviceSetData();    // set RS line to high for data
  
    PMDIN1 =  value;    // Put the value into the register  
    PMPWaitBusy();

    DeviceDeselect();   // disable S1D13517

}

/*********************************************************************
* Function:  BYTE GetReg(WORD index)
* PreCondition: none
* Input: index - register number
* Output: none
* Side Effects: none
* Overview: returns graphics controller register value (byte access)
* Note: none
********************************************************************/
BYTE GetReg(BYTE index)
{

     WORD    value;

    DeviceSetCommand(); // set RS line to low for command
    DeviceSelect();     // enable S1D13517

    DeviceWrite(index);

	DeviceSetData();    // set RS line to high for data

	value = DeviceRead();
	value = DeviceRead();

	DeviceDeselect();   // disable S1D13517

	return (value);
}

/*********************************************************************
* Function:  void ResetDevice()
* PreCondition: none
* Input: none
* Output: none
* Side Effects: none
* Overview: resets LCD, initializes PMP
* Note: none
********************************************************************/
void ResetDevice(void) {
	initSD(); //initializes EADOG 102-64, clears VMAP and then clears EADOG display
	
}	
//void ResetDevice(void)
//{
//   
//    /////////////////////////////////////////////////////////////////////
//    // Initialize the device
//    /////////////////////////////////////////////////////////////////////
//	DeviceInit();
//
//    // WQVGA
//        SetReg(REG2A_DSP_MODE,0x00);
//        SetReg(REG68_POWER_SAVE,0x00);
//        SetReg(REG04_PLL_DDIVIDER,0x17);  
//        SetReg(REG06_PLL_0,0x51);
//        SetReg(REG08_PLL_1,0x00);  
//        SetReg(REG0C_PLL_NDIVIDER,0x7f);
//        SetReg(REG12_CLK_SRC_SELECT,0x01); 
//        SetReg(REG04_PLL_DDIVIDER,0x8b);
//        SetReg(REG0E_SS_CONTROL_0,0x3F);  
//        DelayMs(100);
//        SetReg(REG12_CLK_SRC_SELECT,0x01);
//        SetReg(REG14_LCD_PANEL_TYPE,0x04);       
//        SetReg(REG16_HDISP_WIDTH,0x3b);          
//        SetReg(REG18_HNDP_PERIOD,0x16);           
//        SetReg(REG1A_VDISP_HEIGHT_0,0x0F);
//        SetReg(REG1C_VDISP_HEIGHT_1,0x01);
//        SetReg(REG1E_VNDP_PERIOD,0x06);          
//        SetReg(REG20_PHS_PULSE_WIDTH,0x26);     
//        SetReg(REG22_PHS_PULSE_START,0x03);     
//        SetReg(REG24_PVS_PULSE_WIDTH,0x09);     
//        SetReg(REG26_PVS_PULSE_START,0x02);     
// 
//        SetReg(REG5A_WRITE_WIN_X_SP,0x00);       // Write Window X Start Position Register
//        SetReg(REG5C_WRITE_WIN_Y_SP_0,0x00);     // Write Window Start Position Register 0
//        SetReg(REG5E_WRITE_WIN_Y_SP_1,0x00);     // Write Window Start Position Register 1
//        SetReg(REG60_WRITE_WIN_X_EP,0x76);       // Write Window X End Position Register
//        SetReg(REG62_WRITE_WIN_Y_EP_0,0x43);     // Write Window Y End Position Register 0
//        SetReg(REG64_WRITE_WIN_Y_EP_1,0x03);     // Write Window Y End Position Register 1
// 
//        SetReg(REG28_PCLK_POLARITY,0x00);       //Polarity
//        SetReg(REG82_SDRAM_CONTROL_0,0x03);
//        SetReg(REG8C_SDRAM_RFS_CNT_0,0xFF);
//        SetReg(REG8E_SDRAM_RFS_CNT_1,0x03);
//        SetReg(REG90_SDRAM_BUF_SIZE,0x39);
//        SetReg(REG68_POWER_SAVE,0xE8);
//        SetReg(REG68_POWER_SAVE,0x00);
//        SetReg(REG68_POWER_SAVE,0x01);
//        SetReg(REG84_SDRAM_STATUS_0,0x82);
//
//        SetReg(REG52_INPUT_MODE,0x08);        //Transparency On = 0x08
//        SetReg(REG54_TRANSP_KEY_RED, RED8(TRANSPARENTCOLOR));   //Set To Black
//        SetReg(REG56_TRANSP_KEY_GREEN, GREEN8(TRANSPARENTCOLOR)); //Set To Black
//        SetReg(REG58_TRANSP_KEY_BLUE,BLUE8(TRANSPARENTCOLOR));   //Set To Black
//        SetReg(REG6E_GPO_1,0x07);   //GPO All High
//        SetReg(REG70_PWM_CONTROL,0x85);   //PWM
//        SetReg(REG2A_DSP_MODE,0x01);   //Color Bar Test 0F
//
//        SetReg(REG50_DISPLAY_CONTROL,0x80);   //Refresh
//
//        DelayMs(500);             //THis is needed for SDRAM to stabilize
//
//    /////////////////////////////////////////////////////////////////////
//    // Panel TCON Programming
//    /////////////////////////////////////////////////////////////////////
////    TCON_Init();
//}
//
/*********************************************************************
* Function: void PutPixel(SHORT x, SHORT y)
* PreCondition: none
* Input: x,y - pixel coordinates
* Output: none
* Side Effects: none
* Overview: puts pixel
* Note: none
********************************************************************/



void PutPixel(SHORT x, SHORT y) {
SHORT mytemp;
	CS =0;
	if ( (GetColor()==BLACK) ||(GetColor()==WHITE)) { //if color is not 0x0000 (BLACK) then bail!
	 Plot ((SHORT) x, (SHORT) y); //sets pixel in appropiate pixel bit in VMAP
	 mytemp=Vmap[x] [y/8]; //retreive byte from Vmap
	goto_xy(x,y/8); //address col/page of byte
	 setAO();		//data mode
	 	writeSPI(mytemp);
	}

	
}	
//void PutPixel(SHORT x, SHORT y)
//{
//  static  BYTE i;
//  static  BYTE xc=0;
//
//    xc = x % 8;    //Get the offset of the 8 pixel window      
//  
//    DeviceSelect();     // enable S1D13517
//  
//    DeviceSetCommand(); // set RS line to low for command 
// 
//    PMDIN1 =  REG5A_WRITE_WIN_X_SP;   
//    PMPWaitBusy();     
// 
//	DeviceSetData();    // set RS line to high for data
// 
//      PMDIN1 =  (x >>2);    //Setup the window size/position   
//	  PMPWaitBusy(); 
//      PMDIN1 =  (y >> 2);   
//	  PMPWaitBusy(); 
//      PMDIN1 =  (y);   
//	  PMPWaitBusy(); 
// 
//      DeviceSetCommand(); // set RS line to low for command       
// 
//      PMDIN1 =  (REG66_MEM_DATA_PORT_0);   //Call out the first SDRAM write register   
//	  PMPWaitBusy(); 
//
//      DeviceSetData();    // set RS line to high for data
// 
//      WriteBuffer[xc] = _color;      //Set the pixel of the 8 pixel buffer
//
//      for(i=0;i<8;i++)    // Write 8 pixels, 1 pixel is the color, the other 7 are transparent
//      {
//           DeviceWrite(WriteBuffer[i]); //Output the writebuffer
//      }
//
//      WriteBuffer[xc] = TRANSPARENTCOLOR;  //Make the written pixel transparent again
//      DeviceDeselect();     // disable S1D13517
//
//}
//
/*********************************************************************
* Function: WORD GetPixel(SHORT x, SHORT y)
* PreCondition: none
* Input: x,y - pixel coordinates 
* Output: pixel color
* Side Effects: none
* Overview: returns pixel color at x,y position
* Note: none
********************************************************************/
WORD GetPixel(SHORT x, SHORT y)
{
  //Impossible to do with the Epson S1D13517
}

#ifdef USE_DRV_LINE

/*********************************************************************
* Function: WORD Line(SHORT x1, SHORT y1, SHORT x2, SHORT y2)
*
* PreCondition: none
*
* Input: x1,y1 - starting coordinates, x2,y2 - ending coordinates
*
* Output: For NON-Blocking configuration:
*         - Returns 0 when device is busy and the shape is not yet completely drawn.
*         - Returns 1 when the shape is completely drawn.
*         For Blocking configuration:
*         - Always return 1.
*
* Side Effects: none
*
* Overview: draws line
*
* Note: none
*
********************************************************************/
WORD Line(SHORT x1, SHORT y1, SHORT x2, SHORT y2)
{

 /*
This function has not been completed yet. 
*/

    return (1);
}

#endif
#ifdef USE_DRV_BAR

/*********************************************************************
* Function: WORD Bar(SHORT left, SHORT top, SHORT right, SHORT bottom)
*
* PreCondition: none
*
* Input: left,top - top left corner coordinates,
*        right,bottom - bottom right corner coordinates
*
* Output: For NON-Blocking configuration:
*         - Returns 0 when device is busy and the shape is not yet completely drawn.
*         - Returns 1 when the shape is completely drawn.
*         For Blocking configuration:
*         - Always return 1.
*
* Side Effects: none
*
* Overview: draws rectangle filled with current color
*
* Note: none
*
********************************************************************/
WORD Bar(SHORT left, SHORT top, SHORT right, SHORT bottom)
{

WORD xdelta,ydelta,xstart,xend,loffset,roffset;
      
      if(left>0) loffset = left % 8;
      else loffset = 0;
      roffset = right % 8;
      roffset = 8 - roffset;

      xdelta = right - left;
      ydelta = bottom - top;
      xstart = (left) >> 3;xstart = xstart << 1; 
      xend   = ((right+roffset)-8) >> 3; xend = xend << 1; 
   
      DeviceSelect();     // enable S1D13517

      DeviceSetCommand(); // set RS line to low for command       

    	PMDIN1 =  REG5A_WRITE_WIN_X_SP;   
	    PMPWaitBusy();

	DeviceSetData();    // set RS line to high for data

	PMDIN1 =  xstart;
    PMPWaitBusy();
    PMDIN1 =  top >> 2;
    PMPWaitBusy();
    PMDIN1 =  top;
    PMPWaitBusy();
    PMDIN1 =  xend;
    PMPWaitBusy();
  
 
      DeviceSetCommand(); // set RS line to low for command       

      PMDIN1 =  REG66_MEM_DATA_PORT_0;   
      PMPWaitBusy();

	DeviceSetData();    // set RS line to high for data

     WORD j,i;

     for(i=0;i <= ydelta; i++)  {  
       for(j=0; j < (xdelta + loffset + roffset); j++)
                   {                 

         if((j<loffset) || (j > (xdelta+loffset))) {DeviceWrite(TRANSPARENTCOLOR);}
         else {DeviceWrite(_color);}
                                   
                   }
}

 DeviceDeselect();     // disable S1D13517


  return (1);
}

#endif
#ifdef USE_DRV_CIRCLE

/***************************************************************************
* Function: WORD Circle(SHORT x, SHORT y, SHORT radius)
*
* Overview: This macro draws a circle with the given center and radius.
*
* Input: x - Center x position. 
*		 y - Center y position.
*		 radius - the radius of the circle.
*
* Output: For NON-Blocking configuration:
*         - Returns 0 when device is busy and the shape is not yet completely drawn.
*         - Returns 1 when the shape is completely drawn.
*         For Blocking configuration:
*         - Always return 1.
*
* Side Effects: none
*
********************************************************************/
WORD Circle(SHORT x, SHORT y, SHORT radius)
{

 /*
This function has not been completed yet. 
*/
   
}

#endif
#ifdef USE_DRV_CLEARDEVICE

/*********************************************************************
* Function: void ClearDevice(void)
* PreCondition: none
* Input: none
* Output: none
* Side Effects: none
* Overview: clears screen with current color 
* Note: none
********************************************************************/
void ClearDevice(void)
{
    DeviceSelect();     // enable S1D13517

    DeviceSetCommand(); // set RS line to low for command       

    PMDIN1 =  REG5A_WRITE_WIN_X_SP;   
	PMPWaitBusy();

	DeviceSetData();    // set RS line to high for data

	PMDIN1 =  0x00;     //Set starting window coordinates  
	PMPWaitBusy();
    PMDIN1 =  0x00;   
	PMPWaitBusy();
    PMDIN1 =  0x00;   
	PMPWaitBusy();
    PMDIN1 =  0x76;     //Set starting window coordinates  
	PMPWaitBusy();
    PMDIN1 =  0x43;   
	PMPWaitBusy();
    PMDIN1 =  0x03;   
	PMPWaitBusy();

     WORD j,i;

     for(i=0;i< DISP_VER_RESOLUTION; i++)    
       for(j=0;j< DISP_HOR_RESOLUTION; j++)
                   {                                 
                                   DeviceWrite(_color);
                   }


 DeviceDeselect();     // disable S1D13517

}

#endif




#ifdef USE_DRV_PUTIMAGE
/*********************************************************************
* Function: WORD PutImage(SHORT left, SHORT top, void* bitmap, BYTE stretch)
*
* PreCondition: none
*
* Input: left,top - left top image corner,
*        bitmap - image pointer,
*        stretch - image stretch factor
*
* Output: For NON-Blocking configuration:
*         - Returns 0 when device is busy and the image is not yet completely drawn.
*         - Returns 1 when the image is completely drawn.
*         For Blocking configuration:
*         - Always return 1.
*
* Side Effects: none
*
* Overview: outputs image starting from left,top coordinates
*
* Note: image must be located in flash
*
********************************************************************/
WORD PutImage(SHORT left, SHORT top, void *bitmap, BYTE stretch)
{
    FLASH_BYTE  *flashAddress;
    BYTE        colorDepth;
    WORD        colorTemp;

    // Save current color
    colorTemp = GetColor();

    switch(*((SHORT *)bitmap))
    {
                #ifdef USE_BITMAP_FLASH

        case FLASH:

            // Image address
            flashAddress = ((BITMAP_FLASH *)bitmap)->address;

            // Read color depth
            colorDepth = *(flashAddress + 1);

            // Draw picture
            switch(colorDepth)
            {
                case 1:     PutImage1BPP(left, top, flashAddress, stretch); break; 
                
                    #if (COLOR_DEPTH >= 4)
                case 4:     PutImage4BPP(left, top, flashAddress, stretch); break; 
                    #endif
                    
                    #if (COLOR_DEPTH >= 8)
                case 8:     PutImage8BPP(left, top, flashAddress, stretch); break;
                    #endif
                    
                    #if (COLOR_DEPTH == 16)
                case 16:    PutImage16BPP(left, top, flashAddress, stretch); break;
                    #endif
                    
                default:    break;
            }

            break;
                #endif
                
                #ifdef USE_BITMAP_EXTERNAL

        case EXTERNAL:

            // Get color depth
            ExternalMemoryCallback(bitmap, 1, 1, &colorDepth);

            // Draw picture
            switch(colorDepth)
            {
                case 1:     PutImage1BPPExt(left, top, bitmap, stretch); break;
                
                    #if (COLOR_DEPTH >= 4)
                case 4:     PutImage4BPPExt(left, top, bitmap, stretch); break;
                    #endif
                    
                    #if (COLOR_DEPTH >= 8)
                case 8:     PutImage8BPPExt(left, top, bitmap, stretch); break;
                    #endif
                    
                    #if (COLOR_DEPTH == 16)
                case 16:    PutImage16BPPExt(left, top, bitmap, stretch); break;
                    #endif
                    
                default:    break;
            }

            break;
                #endif

        default:
            break;
    }

    // Restore current color
    SetColor(colorTemp);
    return (1);
}


/*********************************************************************
* Function: void PutImage4BPP(SHORT left, SHORT top, FLASH_BYTE* bitmap, BYTE stretch)
*
* PreCondition: none
*
* Input: left,top - left top image corner, bitmap - image pointer,
*        stretch - image stretch factor
*
* Output: none
*
* Side Effects: none
*
* Overview: outputs 16 color image starting from left,top coordinates
*
* Note: image must be located in flash
*
********************************************************************/
void PutImage4BPP(SHORT left, SHORT top, FLASH_BYTE *bitmap, BYTE stretch)
{

    register FLASH_BYTE *flashAddress;
    register FLASH_BYTE *tempFlashAddress;
    WORD                sizeX, sizeY;
    register WORD       x, y;
    WORD                xc, yc;
    BYTE                temp = 0;

    WORD                pallete[16];
    WORD                counter;

    WORD right,xstart,xend,loffset,roffset;

    // Move pointer to size information
    flashAddress = bitmap + 2;

    // Read image size
    sizeY = *((FLASH_WORD *)flashAddress);
    flashAddress += 2;
    sizeX = *((FLASH_WORD *)flashAddress);
    flashAddress += 2;
  
      right = left + sizeX;
      if(left>0) loffset = left % 8;
      else loffset = 0;
      roffset = right % 8;
      roffset = 8 - roffset;

      xstart = (left) >> 3;xstart = xstart << 1; 
      xend   = ((right+roffset)-8) >> 3; xend = xend << 1; 
   
      DeviceSelect();     // enable S1D13517

      DeviceSetCommand(); // set RS line to low for command       

    	PMDIN1 =  REG5A_WRITE_WIN_X_SP;   
	    PMPWaitBusy();

	DeviceSetData();    // set RS line to high for data

	PMDIN1 =  xstart;PMPWaitBusy();
    PMDIN1 =  top >> 2;PMPWaitBusy();
    PMDIN1 =  top;PMPWaitBusy();
    PMDIN1 =  xend;PMPWaitBusy();
  
      DeviceSetCommand(); // set RS line to low for command       

      PMDIN1 =  REG66_MEM_DATA_PORT_0;   
      PMPWaitBusy();

	DeviceSetData();    // set RS line to high for data


    // Read pallete
    for(counter = 0; counter < 16; counter++)
    {
        pallete[counter] = *((FLASH_WORD *)flashAddress);
        flashAddress += 2;
    }

    for(y = 0; y <= sizeY; y++)
    {
        tempFlashAddress = flashAddress;
        
            flashAddress = tempFlashAddress;
            
            for(x = 0; x < sizeX + loffset + roffset; x++)
            {
              if((x<loffset) || (x > (sizeX+loffset))) {DeviceWrite(TRANSPARENTCOLOR);}
              else{ 
                // Read 2 pixels from flash
                if(x & 0x0001)
                {

                    // second pixel in byte
                // Set color
                #ifdef USE_PALETTE
                    SetColor(temp >> 4);
                #else
                    SetColor(pallete[temp >> 4]);
                #endif
                }
                else
                {
                    temp = *flashAddress;
                    flashAddress++;

                    // first pixel in byte
                // Set color
                #ifdef USE_PALETTE
                    SetColor(temp & 0x0f);
                #else
                    SetColor(pallete[temp & 0x0f]);
                #endif
                }

                // Write pixel to screen
                
                    DeviceWrite(_color);
                
            }
            }

        
    }
}

/*********************************************************************
* Function: void PutImage8BPP(SHORT left, SHORT top, FLASH_BYTE* bitmap, BYTE stretch)
*
* PreCondition: none
*
* Input: left,top - left top image corner, bitmap - image pointer,
*        stretch - image stretch factor
*
* Output: none
*
* Side Effects: none
*
* Overview: outputs 256 color image starting from left,top coordinates
*
* Note: image must be located in flash
*
********************************************************************/
void PutImage8BPP(SHORT left, SHORT top, FLASH_BYTE *bitmap, BYTE stretch)
{
    register FLASH_BYTE *flashAddress;
    register FLASH_BYTE *tempFlashAddress;
    WORD                sizeX, sizeY;
    WORD                x, y;
    WORD                xc, yc;
    BYTE                temp;
    BYTE                stretchX, stretchY;
    WORD                pallete[256];
    WORD                counter;
    WORD right,xstart,xend,loffset,roffset;

    // Move pointer to size information
    flashAddress = bitmap + 2;

    // Read image size
    sizeY = *((FLASH_WORD *)flashAddress);
    flashAddress += 2;
    sizeX = *((FLASH_WORD *)flashAddress)-1;
    flashAddress += 2;
  
       right = left + sizeX;
      if(left>0) loffset = left % 8;
      else loffset = 0;
      roffset = right % 8;
      roffset = 8 - roffset;

      xstart = (left) >> 3;xstart = xstart << 1; 
      xend   = ((right+roffset)-8) >> 3; xend = xend << 1; 
   
      DeviceSelect();     // enable S1D13517

      DeviceSetCommand(); // set RS line to low for command       

    	PMDIN1 =  REG5A_WRITE_WIN_X_SP;   
	    PMPWaitBusy();

	DeviceSetData();    // set RS line to high for data

	PMDIN1 =  xstart;PMPWaitBusy();
    PMDIN1 =  top >> 2;PMPWaitBusy();
    PMDIN1 =  top;PMPWaitBusy();
    PMDIN1 =  xend;PMPWaitBusy();
  
      DeviceSetCommand(); // set RS line to low for command       

      PMDIN1 =  REG66_MEM_DATA_PORT_0;   
      PMPWaitBusy();

	DeviceSetData();    // set RS line to high for data

    // Read pallete
    for(counter = 0; counter < 256; counter++)
    {
        pallete[counter] = *((FLASH_WORD *)flashAddress);
        flashAddress += 2;
    }

    for(y = 0; y <= sizeY; y++)
    {
        tempFlashAddress = flashAddress;

        for(stretchY = 0; stretchY < stretch; stretchY++)
        {
          
            flashAddress = tempFlashAddress;

              for(x = 0; x < sizeX + loffset + roffset; x++)
            {
              if((x<loffset) || (x > (sizeX+loffset))) {DeviceWrite(TRANSPARENTCOLOR);}
              else{ 

                // Read pixels from flash
                temp = *flashAddress;
                flashAddress++;

                // Set color
            #ifdef USE_PALETTE
                SetColor(temp);
            #else
                SetColor(pallete[temp]);
            #endif

                // Write pixel to screen
                for(stretchX = 0; stretchX < stretch; stretchX++)
                {
                    DeviceWrite(_color);
                }
            }
               }

        }
    }
}

/*********************************************************************
* Function: void PutImage16BPP(SHORT left, SHORT top, FLASH_BYTE* bitmap, BYTE stretch)
*
* PreCondition: none
*
* Input: left,top - left top image corner, bitmap - image pointer,
*        stretch - image stretch factor
*
* Output: none
*
* Side Effects: none
*
* Overview: outputs hicolor image starting from left,top coordinates
*
* Note: image must be located in flash
*
********************************************************************/
void PutImage16BPP(SHORT left, SHORT top, FLASH_BYTE *bitmap, BYTE stretch)
{
    register FLASH_WORD *flashAddress;
    register FLASH_WORD *tempFlashAddress;
    WORD                sizeX, sizeY;
    register WORD       x, y;
    WORD                xc, yc;
    WORD                temp;
    register BYTE       stretchX, stretchY;
    WORD right,xstart,xend,loffset,roffset;
    // Move pointer to size information
    flashAddress = (FLASH_WORD *)bitmap + 1;

    // Read image size
    sizeY = *flashAddress;
    flashAddress++;
    sizeX = *flashAddress-1;
    flashAddress++;


           right = left + sizeX;
      if(left>0) loffset = left % 8;
      else loffset = 0;
      roffset = right % 8;
      roffset = 8 - roffset;

      xstart = (left) >> 3;xstart = xstart << 1; 
      xend   = ((right+roffset)-8) >> 3; xend = xend << 1; 
   
      DeviceSelect();     // enable S1D13517

      DeviceSetCommand(); // set RS line to low for command       

    	PMDIN1 =  REG5A_WRITE_WIN_X_SP;   
	    PMPWaitBusy();

	DeviceSetData();    // set RS line to high for data

	PMDIN1 =  xstart;PMPWaitBusy();
    PMDIN1 =  top >> 2;PMPWaitBusy();
    PMDIN1 =  top;PMPWaitBusy();
    PMDIN1 =  xend;PMPWaitBusy();
  
      DeviceSetCommand(); // set RS line to low for command       

      PMDIN1 =  REG66_MEM_DATA_PORT_0;   
      PMPWaitBusy();

	DeviceSetData();    // set RS line to high for data

    yc = top;
    for(y = 0; y <= sizeY; y++)
    {
        tempFlashAddress = flashAddress;
        for(stretchY = 0; stretchY < stretch; stretchY++)
        {
            flashAddress = tempFlashAddress;
            xc = left;
             for(x = 0; x < sizeX + loffset + roffset; x++)
            {
              if((x<loffset) || (x > (sizeX+loffset))) {DeviceWrite(TRANSPARENTCOLOR);}
              else{ 


                // Read pixels from flash
                temp = *flashAddress;
                flashAddress++;

                // Set color
                SetColor(temp);

                // Write pixel to screen
                for(stretchX = 0; stretchX < stretch; stretchX++)
                {
                    DeviceWrite(_color);
                }
              }
            }

            yc++;
        }
    }
}


#endif
#ifdef USE_BITMAP_EXTERNAL

/*********************************************************************
* Function: void PutImage1BPPExt(SHORT left, SHORT top, void* bitmap, BYTE stretch)
*
* PreCondition: none
*
* Input: left,top - left top image corner, bitmap - image pointer,
*        stretch - image stretch factor
*
* Output: none
*
* Side Effects: none
*
* Overview: outputs monochrome image starting from left,top coordinates
*
* Note: image must be located in external memory
*
********************************************************************/
void PutImage1BPPExt(SHORT left, SHORT top, void *bitmap, BYTE stretch)
{

 /*
This function has not been completed yet. 
*/

}

/*********************************************************************
* Function: void PutImage4BPPExt(SHORT left, SHORT top, void* bitmap, BYTE stretch)
*
* PreCondition: none
*
* Input: left,top - left top image corner, bitmap - image pointer,
*        stretch - image stretch factor
*
* Output: none
*
* Side Effects: none
*
* Overview: outputs monochrome image starting from left,top coordinates
*
* Note: image must be located in external memory
*
********************************************************************/
void PutImage4BPPExt(SHORT left, SHORT top, void *bitmap, BYTE stretch)
{

/*
This function has not been completed yet. 
*/

}

/*********************************************************************
* Function: void PutImage8BPPExt(SHORT left, SHORT top, void* bitmap, BYTE stretch)
*
* PreCondition: none
*
* Input: left,top - left top image corner, bitmap - image pointer,
*        stretch - image stretch factor
*
* Output: none
*
* Side Effects: none
*
* Overview: outputs monochrome image starting from left,top coordinates
*
* Note: image must be located in external memory
*
********************************************************************/
void PutImage8BPPExt(SHORT left, SHORT top, void *bitmap, BYTE stretch)
{

/*
This function has not been completed yet. 
*/

}

/*********************************************************************
* Function: void PutImage16BPPExt(SHORT left, SHORT top, void* bitmap, BYTE stretch)
*
* PreCondition: none
*
* Input: left,top - left top image corner, bitmap - image pointer,
*        stretch - image stretch factor
*
* Output: none
*
* Side Effects: none
*
* Overview: outputs monochrome image starting from left,top coordinates
*
* Note: image must be located in external memory
*
********************************************************************/
void PutImage16BPPExt(SHORT left, SHORT top, void *bitmap, BYTE stretch)
{

/*
This function has not been completed yet. 
*/

}

#endif


